from unittest import TestCase
from keras_transformer.transformer import _get_max_suffix_repeat_times


class TestSuffixRepeat(TestCase):

    def test_abcd(self):
        self.assertEqual(1, _get_max_suffix_repeat_times('abcdabcdabcd', max_len=3))
        self.assertEqual(1, _get_max_suffix_repeat_times('abcdabcdabcd', max_len=6))
        self.assertEqual(2, _get_max_suffix_repeat_times('abcdabcdabcd', max_len=11))
        self.assertEqual(3, _get_max_suffix_repeat_times('abcdabcdabcd', max_len=12))
        self.assertEqual(3, _get_max_suffix_repeat_times('abcdabcdabcd', max_len=16))
        self.assertEqual(2, _get_max_suffix_repeat_times('bcdabcdabcd', max_len=16))



# encoding: utf-8
from __future__ import unicode_literals

import unittest
import numpy as np
from keras_transformer import get_model, decode


class TestTranslate(unittest.TestCase):

    @staticmethod
    def _build_token_dict(token_list):
        token_dict = {
            '<PAD>': 0,
            '<START>': 1,
            '<END>': 2,
        }
        for tokens in token_list:
            for token in tokens:
                if token not in token_dict:
                    token_dict[token] = len(token_dict)
        return token_dict

    def test_translate(self):
        source_tokens = [
            'i need more power'.split(' '),
            'eat jujube and pill'.split(' '),
        ]
        target_tokens = [
            list('我要更多的抛瓦'),
            list('吃枣💊'),
        ]

        # Generate dictionaries
        source_token_dict = self._build_token_dict(source_tokens)
        target_token_dict = self._build_token_dict(target_tokens)
        target_token_dict_inv = {v: k for k, v in target_token_dict.items()}

        # Add special tokens
        encode_tokens = [['<START>'] + tokens + ['<END>'] for tokens in source_tokens]
        decode_tokens = [['<START>'] + tokens + ['<END>'] for tokens in target_tokens]
        output_tokens = [tokens + ['<END>', '<PAD>'] for tokens in target_tokens]

        # Padding
        source_max_len = max(map(len, encode_tokens))
        target_max_len = max(map(len, decode_tokens))

        encode_tokens = [tokens + ['<PAD>'] * (source_max_len - len(tokens)) for tokens in encode_tokens]
        decode_tokens = [tokens + ['<PAD>'] * (target_max_len - len(tokens)) for tokens in decode_tokens]
        output_tokens = [tokens + ['<PAD>'] * (target_max_len - len(tokens)) for tokens in output_tokens]

        encode_input = [list(map(lambda x: source_token_dict[x], tokens)) for tokens in encode_tokens]
        decode_input = [list(map(lambda x: target_token_dict[x], tokens)) for tokens in decode_tokens]
        decode_output = [list(map(lambda x: [target_token_dict[x]], tokens)) for tokens in output_tokens]

        # Build & fit model
        model = get_model(
            token_num=max(len(source_token_dict), len(target_token_dict)),
            embed_dim=32,
            encoder_num=2,
            decoder_num=2,
            head_num=4,
            hidden_dim=128,
            dropout_rate=0.05,
            use_same_embed=False,  # Use different embeddings for different languages
        )
        model.compile('adam', 'sparse_categorical_crossentropy')
        model.summary()
        model.fit(
            x=[np.array(encode_input * 1024), np.array(decode_input * 1024)],
            y=np.array(decode_output * 1024),
            epochs=10,
            batch_size=32,
        )

        # Predict
        decoded = decode(
            model,
            encode_input,
            start_token=target_token_dict['<START>'],
            end_token=target_token_dict['<END>'],
            pad_token=target_token_dict['<PAD>'],
        )
        for i in range(len(encode_input)):
            predicted = ''.join(map(lambda x: target_token_dict_inv[x], decoded[i][1:-1]))
            self.assertEqual(''.join(target_tokens[i]), predicted)



import math
from .backend import backend as K


def gelu(x):
    """An approximation of gelu.

    See: https://arxiv.org/pdf/1606.08415.pdf
    """
    return 0.5 * x * (1.0 + K.tanh(math.sqrt(2.0 / math.pi) * (x + 0.044715 * x * x * x)))



import unittest
from keras_transformer.backend import backend as K
from keras_transformer import gelu


class TestGelu(unittest.TestCase):

    def test_sample(self):
        try:
            results = gelu(K.constant([-30.0, -1.0, 0.0, 1.0, 30.0])).eval(session=K.get_session())
        except Exception as e:
            try:
                results = gelu(K.constant([-30.0, -1.0, 0.0, 1.0, 30.0])).eval()
            except Exception as e:
                results = gelu(K.constant([-30.0, -1.0, 0.0, 1.0, 30.0])).numpy()
        self.assertEqual(0.0, results[0])
        self.assertGreater(0.0, results[1])
        self.assertLess(-1.0, results[1])
        self.assertEqual(0.0, results[2])
        self.assertGreater(1.0, results[3])
        self.assertLess(0.0, results[3])
        self.assertEqual(30.0, results[4])



import os
import tempfile
import unittest
import numpy as np
from keras_transformer.backend import keras
from keras_transformer import get_custom_objects, get_model


class TestGetModel(unittest.TestCase):

    def test_get_same(self):
        model = get_model(
            token_num=13,
            embed_dim=30,
            encoder_num=3,
            decoder_num=2,
            head_num=3,
            hidden_dim=120,
            attention_activation=None,
            feed_forward_activation='relu',
            dropout_rate=0.05,
            use_same_embed=True,
            embed_weights=np.random.random((13, 30)),
            trainable=False,
        )
        model.compile(
            optimizer=keras.optimizers.Adam(),
            loss=keras.losses.categorical_crossentropy,
            metrics={},
        )
        model_path = os.path.join(tempfile.gettempdir(), 'test_transformer_%f.h5' % np.random.random())
        model.save(model_path)
        model = keras.models.load_model(model_path, custom_objects=get_custom_objects())
        model.summary()
        try:
            keras.utils.plot_model(model, 'transformer_same.png')
        except Exception as e:
            print(e)
        self.assertIsNotNone(model)

    def test_get_diff(self):
        model = get_model(
            token_num=[13, 14],
            embed_dim=30,
            encoder_num=3,
            decoder_num=2,
            head_num=3,
            hidden_dim=120,
            attention_activation=None,
            feed_forward_activation='relu',
            dropout_rate=0.05,
            use_same_embed=False,
        )
        model.compile(
            optimizer=keras.optimizers.Adam(),
            loss=keras.losses.categorical_crossentropy,
            metrics={},
        )
        model_path = os.path.join(tempfile.gettempdir(), 'test_transformer_%f.h5' % np.random.random())
        model.save(model_path)
        model = keras.models.load_model(model_path, custom_objects=get_custom_objects())
        model.summary()
        try:
            keras.utils.plot_model(model, 'transformer_diff.png')
        except Exception as e:
            print(e)
        self.assertIsNotNone(model)

    def test_get_adapter(self):
        model = get_model(
            token_num=[13, 14],
            embed_dim=30,
            encoder_num=3,
            decoder_num=2,
            head_num=3,
            hidden_dim=120,
            attention_activation=None,
            feed_forward_activation='relu',
            dropout_rate=0.05,
            use_same_embed=False,
            use_adapter=True,
            adapter_units=3,
            adapter_activation='relu',
        )
        model.compile(
            optimizer=keras.optimizers.Adam(),
            loss=keras.losses.categorical_crossentropy,
            metrics={},
        )
        model_path = os.path.join(tempfile.gettempdir(), 'test_transformer_%f.h5' % np.random.random())
        model.save(model_path)
        model = keras.models.load_model(model_path, custom_objects=get_custom_objects())
        model.summary()
        try:
            keras.utils.plot_model(model, 'transformer_adapter.png')
        except Exception as e:
            print(e)
        self.assertIsNotNone(model)



import os

__all__ = [
    'keras', 'utils', 'activations', 'applications', 'backend', 'datasets', 'engine',
    'layers', 'preprocessing', 'wrappers', 'callbacks', 'constraints', 'initializers',
    'metrics', 'models', 'losses', 'optimizers', 'regularizers', 'TF_KERAS', 'EAGER_MODE'
]

TF_KERAS = False
EAGER_MODE = False

if 'TF_KERAS' in os.environ and os.environ['TF_KERAS'] != '0':
    import tensorflow as tf
    from tensorflow.python import keras
    TF_KERAS = True
    if 'TF_EAGER' in os.environ and os.environ['TF_EAGER'] != '0':
        if int(tf.version.VERSION.split('.')[0]) < 2:
            import tensorflow as tf
            tf.enable_eager_execution()
        EAGER_MODE = True
else:
    import keras

utils = keras.utils
activations = keras.activations
applications = keras.applications
backend = keras.backend
datasets = keras.datasets
engine = keras.engine
layers = keras.layers
preprocessing = keras.preprocessing
wrappers = keras.wrappers
callbacks = keras.callbacks
constraints = keras.constraints
initializers = keras.initializers
metrics = keras.metrics
models = keras.models
losses = keras.losses
optimizers = keras.optimizers
regularizers = keras.regularizers
